@@ -28,29 +28,29 @@
 
 #include <gdbus.h>
 
+#include <connman/session.h>
+
 #include "connman.h"
 
 static DBusConnection *connection;
 static GHashTable *session_hash;
 static connman_bool_t sessionmode;
-static struct session_info *ecall_info;
+static struct connman_session *ecall_session;
+static struct connman_session_config *session_config;
 
 enum connman_session_trigger {
 	CONNMAN_SESSION_TRIGGER_UNKNOWN		= 0,
 	CONNMAN_SESSION_TRIGGER_SETTING		= 1,
 	CONNMAN_SESSION_TRIGGER_CONNECT		= 2,
 	CONNMAN_SESSION_TRIGGER_DISCONNECT	= 3,
-	CONNMAN_SESSION_TRIGGER_PERIODIC	= 4,
-	CONNMAN_SESSION_TRIGGER_SERVICE		= 5,
-	CONNMAN_SESSION_TRIGGER_ECALL		= 6,
+	CONNMAN_SESSION_TRIGGER_SERVICE		= 4,
+	CONNMAN_SESSION_TRIGGER_ECALL		= 5,
 };
 
 enum connman_session_reason {
 	CONNMAN_SESSION_REASON_UNKNOWN		= 0,
 	CONNMAN_SESSION_REASON_CONNECT		= 1,
-	CONNMAN_SESSION_REASON_DISCONNECT	= 2,
-	CONNMAN_SESSION_REASON_FREE_RIDE	= 3,
-	CONNMAN_SESSION_REASON_PERIODIC		= 4,
+	CONNMAN_SESSION_REASON_FREE_RIDE	= 2,
 };
 
 enum connman_session_state {
@@ -91,12 +91,7 @@ struct session_info {
 	connman_bool_t priority;
 	GSList *allowed_bearers;
 	connman_bool_t avoid_handover;
-	connman_bool_t stay_connected;
-	unsigned int periodic_connect;
-	unsigned int idle_timeout;
-	connman_bool_t ecall;
 	enum connman_session_roaming_policy roaming_policy;
-	unsigned int marker;
 
 	struct service_entry *entry;
 	enum connman_session_reason reason;
@@ -112,6 +107,8 @@ struct connman_session {
 	struct session_info *info;
 	struct session_info *info_last;
 
+	connman_bool_t ecall;
+
 	GSequence *service_list;
 	GHashTable *service_hash;
 };
@@ -133,8 +130,6 @@ static const char *trigger2string(enum connman_session_trigger trigger)
 		return "connect";
 	case CONNMAN_SESSION_TRIGGER_DISCONNECT:
 		return "disconnect";
-	case CONNMAN_SESSION_TRIGGER_PERIODIC:
-		return "periodic";
 	case CONNMAN_SESSION_TRIGGER_SERVICE:
 		return "service";
 	case CONNMAN_SESSION_TRIGGER_ECALL:
@@ -151,12 +146,8 @@ static const char *reason2string(enum connman_session_reason reason)
 		return "unknown";
 	case CONNMAN_SESSION_REASON_CONNECT:
 		return "connect";
-	case CONNMAN_SESSION_REASON_DISCONNECT:
-		return "disconnect";
 	case CONNMAN_SESSION_REASON_FREE_RIDE:
 		return "free-ride";
-	case CONNMAN_SESSION_REASON_PERIODIC:
-		return "periodic";
 	}
 
 	return NULL;
@@ -200,26 +191,6 @@ static enum connman_session_type string2type(const char *type)
 	return CONNMAN_SESSION_TYPE_ANY;
 }
 
-static const char *roamingpolicy2string(enum connman_session_roaming_policy policy)
-{
-	switch (policy) {
-	case CONNMAN_SESSION_ROAMING_POLICY_UNKNOWN:
-		return "unknown";
-	case CONNMAN_SESSION_ROAMING_POLICY_DEFAULT:
-		return "default";
-	case CONNMAN_SESSION_ROAMING_POLICY_ALWAYS:
-		return "always";
-	case CONNMAN_SESSION_ROAMING_POLICY_FORBIDDEN:
-		return "forbidden";
-	case CONNMAN_SESSION_ROAMING_POLICY_NATIONAL:
-		return "national";
-	case CONNMAN_SESSION_ROAMING_POLICY_INTERNATIONAL:
-		return "international";
-	}
-
-	return NULL;
-}
-
 static enum connman_session_roaming_policy string2roamingpolicy(const char *policy)
 {
 	if (g_strcmp0(policy, "default") == 0)
@@ -282,6 +253,26 @@ static char *service2bearer(enum connman_service_type type)
 	return "";
 }
 
+static int config_get_bool(const char *id, const char *key, connman_bool_t *val)
+{
+	if (session_config == NULL) {
+		*val = FALSE;
+		return -EINVAL;
+	}
+
+	return (*session_config->get_bool)(id, key, val);
+}
+
+static int config_get_string(const char *id, const char *key, char **val)
+{
+	if (session_config == NULL) {
+		*val = NULL;
+		return -EINVAL;
+	}
+
+	return (*session_config->get_string)(id, key, val);
+}
+
 static void cleanup_bearer_info(gpointer data, gpointer user_data)
 {
 	struct bearer_info *info = data;
@@ -410,7 +401,6 @@ static void append_notify(DBusMessageIter *dict,
 {
 	struct session_info *info = session->info;
 	struct session_info *info_last = session->info_last;
-	const char *policy;
 	struct connman_service *service;
 	const char *name, *ifname, *bearer;
 
@@ -470,14 +460,6 @@ static void append_notify(DBusMessageIter *dict,
 		info_last->type = info->type;
 	}
 
-	if (session->append_all == TRUE ||
-			info->priority != info_last->priority) {
-		connman_dbus_dict_append_basic(dict, "Priority",
-						DBUS_TYPE_BOOLEAN,
-						&info->priority);
-		info_last->priority = info->priority;
-	}
-
 	if (session->append_all == TRUE ||
 			info->allowed_bearers != info_last->allowed_bearers) {
 		connman_dbus_dict_append_array(dict, "AllowedBearers",
@@ -495,55 +477,6 @@ static void append_notify(DBusMessageIter *dict,
 		info_last->avoid_handover = info->avoid_handover;
 	}
 
-	if (session->append_all == TRUE ||
-			info->stay_connected != info_last->stay_connected) {
-		connman_dbus_dict_append_basic(dict, "StayConnected",
-						DBUS_TYPE_BOOLEAN,
-						&info->stay_connected);
-		info_last->stay_connected = info->stay_connected;
-	}
-
-	if (session->append_all == TRUE ||
-			info->periodic_connect != info_last->periodic_connect) {
-		connman_dbus_dict_append_basic(dict, "PeriodicConnect",
-						DBUS_TYPE_UINT32,
-						&info->periodic_connect);
-		info_last->periodic_connect = info->periodic_connect;
-	}
-
-	if (session->append_all == TRUE ||
-			info->idle_timeout != info_last->idle_timeout) {
-		connman_dbus_dict_append_basic(dict, "IdleTimeout",
-						DBUS_TYPE_UINT32,
-						&info->idle_timeout);
-		info_last->idle_timeout = info->idle_timeout;
-	}
-
-	if (session->append_all == TRUE ||
-			info->ecall != info_last->ecall) {
-		connman_dbus_dict_append_basic(dict, "EmergencyCall",
-						DBUS_TYPE_BOOLEAN,
-						&info->ecall);
-		info_last->ecall = info->ecall;
-	}
-
-	if (session->append_all == TRUE ||
-			info->roaming_policy != info_last->roaming_policy) {
-		policy = roamingpolicy2string(info->roaming_policy);
-		connman_dbus_dict_append_basic(dict, "RoamingPolicy",
-						DBUS_TYPE_STRING,
-						&policy);
-		info_last->roaming_policy = info->roaming_policy;
-	}
-
-	if (session->append_all == TRUE ||
-			info->marker != info_last->marker) {
-		connman_dbus_dict_append_basic(dict, "SessionMarker",
-						DBUS_TYPE_UINT32,
-						&info->marker);
-		info_last->marker = info->marker;
-	}
-
 	session->append_all = FALSE;
 }
 
@@ -584,15 +517,8 @@ static connman_bool_t compute_notifiable_changes(struct connman_session *session
 			info->state >= CONNMAN_SESSION_STATE_CONNECTED)
 		return TRUE;
 
-	if (info->periodic_connect != info_last->periodic_connect ||
-			info->allowed_bearers != info_last->allowed_bearers ||
+	if (info->allowed_bearers != info_last->allowed_bearers ||
 			info->avoid_handover != info_last->avoid_handover ||
-			info->stay_connected != info_last->stay_connected ||
-			info->roaming_policy != info_last->roaming_policy ||
-			info->idle_timeout != info_last->idle_timeout ||
-			info->priority != info_last->priority ||
-			info->marker != info_last->marker ||
-			info->ecall != info_last->ecall ||
 			info->type != info_last->type)
 		return TRUE;
 
@@ -860,10 +786,8 @@ static connman_bool_t explicit_connect(enum connman_session_reason reason)
 	switch (reason) {
 	case CONNMAN_SESSION_REASON_UNKNOWN:
 	case CONNMAN_SESSION_REASON_FREE_RIDE:
-	case CONNMAN_SESSION_REASON_DISCONNECT:
 		break;
 	case CONNMAN_SESSION_REASON_CONNECT:
-	case CONNMAN_SESSION_REASON_PERIODIC:
 		return TRUE;
 	}
 
@@ -888,9 +812,6 @@ static connman_bool_t explicit_disconnect(struct session_info *info)
 	if (__connman_service_session_dec(info->entry->service) == FALSE)
 		return FALSE;
 
-	if (ecall_info != NULL && ecall_info != info)
-		return FALSE;
-
 	return TRUE;
 }
 
@@ -1014,14 +935,13 @@ static void deselect_service(struct session_info *info)
 		pending_timeout_add(0, call_disconnect, entry);
 }
 
-static void deselect_and_disconnect(struct connman_session *session,
-					enum connman_session_reason reason)
+static void deselect_and_disconnect(struct connman_session *session)
 {
 	struct session_info *info = session->info;
 
 	deselect_service(info);
 
-	info->reason = reason;
+	info->reason = CONNMAN_SESSION_REASON_FREE_RIDE;
 }
 
 static void select_connected_service(struct session_info *info,
@@ -1228,7 +1148,7 @@ static void session_changed(struct connman_session *session,
 				 * The currently selected service is
 				 * not part of this session anymore.
 				 */
-				deselect_and_disconnect(session, info->reason);
+				deselect_and_disconnect(session);
 			}
 
 			g_hash_table_remove_all(service_hash_last);
@@ -1239,7 +1159,7 @@ static void session_changed(struct connman_session *session,
 			if (info->state >= CONNMAN_SESSION_STATE_CONNECTED &&
 					is_type_matching_state(&info->state,
 							info->type) == FALSE)
-				deselect_and_disconnect(session, info->reason);
+				deselect_and_disconnect(session);
 		}
 
 		if (info->state == CONNMAN_SESSION_STATE_DISCONNECTED) {
@@ -1248,6 +1168,11 @@ static void session_changed(struct connman_session *session,
 		}
 
 		break;
+	case CONNMAN_SESSION_TRIGGER_ECALL:
+		/*
+		 * For the time beeing we fallback to normal connect
+		 * strategy.
+		 */
 	case CONNMAN_SESSION_TRIGGER_CONNECT:
 		if (info->state >= CONNMAN_SESSION_STATE_CONNECTED) {
 			if (info->entry->reason == CONNMAN_SESSION_REASON_CONNECT)
@@ -1267,19 +1192,7 @@ static void session_changed(struct connman_session *session,
 
 		break;
 	case CONNMAN_SESSION_TRIGGER_DISCONNECT:
-		deselect_and_disconnect(session,
-					CONNMAN_SESSION_REASON_DISCONNECT);
-
-		break;
-	case CONNMAN_SESSION_TRIGGER_PERIODIC:
-		if (info->state >= CONNMAN_SESSION_STATE_CONNECTED) {
-			info->entry->reason = CONNMAN_SESSION_REASON_PERIODIC;
-			__connman_service_session_inc(info->entry->service);
-			break;
-		}
-
-		select_and_connect(session,
-				CONNMAN_SESSION_REASON_PERIODIC);
+		deselect_and_disconnect(session);
 
 		break;
 	case CONNMAN_SESSION_TRIGGER_SERVICE:
@@ -1289,21 +1202,12 @@ static void session_changed(struct connman_session *session,
 			break;
 		}
 
-		deselect_and_disconnect(session, info->reason);
+		deselect_and_disconnect(session);
 
-		if (info->reason == CONNMAN_SESSION_REASON_FREE_RIDE ||
-				info->stay_connected == TRUE) {
+		if (info->reason == CONNMAN_SESSION_REASON_FREE_RIDE) {
 			select_and_connect(session, info->reason);
 		}
 
-		break;
-	case CONNMAN_SESSION_TRIGGER_ECALL:
-		if (info->state == CONNMAN_SESSION_STATE_DISCONNECTED &&
-				info->entry != NULL &&
-				info->entry->service != NULL) {
-			deselect_and_disconnect(session, info->reason);
-		}
-
 		break;
 	}
 
@@ -1314,14 +1218,17 @@ static DBusMessage *connect_session(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_session *session = user_data;
-	struct session_info *info = session->info;
 
 	DBG("session %p", session);
 
-	if (ecall_info != NULL && ecall_info != info)
-		return __connman_error_failed(msg, EBUSY);
+	if (ecall_session != NULL) {
+		if (ecall_session->ecall == TRUE && ecall_session != session)
+			return __connman_error_failed(msg, EBUSY);
 
-	session_changed(session, CONNMAN_SESSION_TRIGGER_CONNECT);
+		session->ecall = TRUE;
+		session_changed(session, CONNMAN_SESSION_TRIGGER_ECALL);
+	} else
+		session_changed(session, CONNMAN_SESSION_TRIGGER_CONNECT);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -1330,70 +1237,21 @@ static DBusMessage *disconnect_session(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_session *session = user_data;
-	struct session_info *info = session->info;
 
 	DBG("session %p", session);
 
-	if (ecall_info != NULL && ecall_info != info)
-		return __connman_error_failed(msg, EBUSY);
+	if (ecall_session != NULL) {
+		if (ecall_session->ecall == TRUE && ecall_session != session)
+			return __connman_error_failed(msg, EBUSY);
+
+		session->ecall = FALSE;
+	}
 
 	session_changed(session, CONNMAN_SESSION_TRIGGER_DISCONNECT);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static void update_ecall_sessions(struct connman_session *session)
-{
-	struct session_info *info = session->info;
-	struct connman_session *session_iter;
-	GHashTableIter iter;
-	gpointer key, value;
-
-	g_hash_table_iter_init(&iter, session_hash);
-
-	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
-		session_iter = value;
-
-		if (session_iter == session)
-			continue;
-
-		session_iter->info->ecall = info->ecall;
-
-		session_changed(session_iter, CONNMAN_SESSION_TRIGGER_ECALL);
-	}
-}
-
-static void update_ecall(struct connman_session *session)
-{
-	struct session_info *info = session->info;
-	struct session_info *info_last = session->info_last;
-
-	DBG("session %p ecall_info %p ecall %d -> %d", session,
-		ecall_info, info_last->ecall, info->ecall);
-
-	if (ecall_info == NULL) {
-		if (!(info_last->ecall == FALSE && info->ecall == TRUE))
-			goto err;
-
-		ecall_info = info;
-	} else if (ecall_info == info) {
-		if (!(info_last->ecall == TRUE && info->ecall == FALSE))
-			goto err;
-
-		ecall_info = NULL;
-	} else {
-		goto err;
-	}
-
-	update_ecall_sessions(session);
-
-	return;
-
-err:
-	/* not a valid transition */
-	info->ecall = info_last->ecall;
-}
-
 static DBusMessage *change_session(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
@@ -1434,31 +1292,9 @@ static DBusMessage *change_session(DBusConnection *conn,
 		}
 		break;
 	case DBUS_TYPE_BOOLEAN:
-		if (g_str_equal(name, "Priority") == TRUE) {
-			dbus_message_iter_get_basic(&value,
-					&info->priority);
-		} else if (g_str_equal(name, "AvoidHandover") == TRUE) {
+		if (g_str_equal(name, "AvoidHandover") == TRUE) {
 			dbus_message_iter_get_basic(&value,
 					&info->avoid_handover);
-		} else if (g_str_equal(name, "StayConnected") == TRUE) {
-			dbus_message_iter_get_basic(&value,
-					&info->stay_connected);
-		} else if (g_str_equal(name, "EmergencyCall") == TRUE) {
-			dbus_message_iter_get_basic(&value,
-					&info->ecall);
-
-			update_ecall(session);
-		} else {
-			goto err;
-		}
-		break;
-	case DBUS_TYPE_UINT32:
-		if (g_str_equal(name, "PeriodicConnect") == TRUE) {
-			dbus_message_iter_get_basic(&value,
-					&info->periodic_connect);
-		} else if (g_str_equal(name, "IdleTimeout") == TRUE) {
-			dbus_message_iter_get_basic(&value,
-					&info->idle_timeout);
 		} else {
 			goto err;
 		}
@@ -1467,10 +1303,6 @@ static DBusMessage *change_session(DBusConnection *conn,
 		if (g_str_equal(name, "ConnectionType") == TRUE) {
 			dbus_message_iter_get_basic(&value, &val);
 			info->type = string2type(val);
-		} else if (g_str_equal(name, "RoamingPolicy") == TRUE) {
-			dbus_message_iter_get_basic(&value, &val);
-			info->roaming_policy =
-					string2roamingpolicy(val);
 		} else {
 			goto err;
 		}
@@ -1522,8 +1354,7 @@ static int session_disconnect(struct connman_session *session)
 	g_dbus_unregister_interface(connection, session->session_path,
 						CONNMAN_SESSION_INTERFACE);
 
-	deselect_and_disconnect(session,
-				CONNMAN_SESSION_REASON_DISCONNECT);
+	deselect_and_disconnect(session);
 
 	g_hash_table_remove(session_hash, session->session_path);
 
@@ -1543,11 +1374,10 @@ static DBusMessage *destroy_session(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_session *session = user_data;
-	struct session_info *info = session->info;
 
 	DBG("session %p", session);
 
-	if (ecall_info != NULL && ecall_info != info)
+	if (ecall_session != NULL && ecall_session != session)
 		return __connman_error_failed(msg, EBUSY);
 
 	session_disconnect(session);
@@ -1575,21 +1405,18 @@ int __connman_session_create(DBusMessage *msg)
 	struct session_info *info, *info_last;
 
 	enum connman_session_type type = CONNMAN_SESSION_TYPE_ANY;
-	connman_bool_t priority = FALSE, avoid_handover = FALSE;
-	connman_bool_t stay_connected = FALSE, ecall = FALSE;
-	enum connman_session_roaming_policy roaming_policy =
-				CONNMAN_SESSION_ROAMING_POLICY_FORBIDDEN;
+	connman_bool_t priority, avoid_handover = FALSE;
+	connman_bool_t ecall_app;
+	enum connman_session_roaming_policy roaming_policy;
+	char *roaming_policy_str;
 	GSList *allowed_bearers = NULL;
-	unsigned int periodic_connect = 0;
-	unsigned int idle_timeout = 0;
-
 	int err;
 
 	owner = dbus_message_get_sender(msg);
 
 	DBG("owner %s", owner);
 
-	if (ecall_info != NULL) {
+	if (ecall_session != NULL && ecall_session->ecall == TRUE) {
 		/*
 		 * If there is an emergency call already going on,
 		 * ignore session creation attempt
@@ -1621,29 +1448,9 @@ int __connman_session_create(DBusMessage *msg)
 			}
 			break;
 		case DBUS_TYPE_BOOLEAN:
-			if (g_str_equal(key, "Priority") == TRUE) {
-				dbus_message_iter_get_basic(&value,
-							&priority);
-			} else if (g_str_equal(key, "AvoidHandover") == TRUE) {
+			if (g_str_equal(key, "AvoidHandover") == TRUE) {
 				dbus_message_iter_get_basic(&value,
 							&avoid_handover);
-			} else if (g_str_equal(key, "StayConnected") == TRUE) {
-				dbus_message_iter_get_basic(&value,
-							&stay_connected);
-			} else if (g_str_equal(key, "EmergencyCall") == TRUE) {
-				dbus_message_iter_get_basic(&value,
-							&ecall);
-			} else {
-				return -EINVAL;
-			}
-			break;
-		case DBUS_TYPE_UINT32:
-			if (g_str_equal(key, "PeriodicConnect") == TRUE) {
-				dbus_message_iter_get_basic(&value,
-							&periodic_connect);
-			} else if (g_str_equal(key, "IdleTimeout") == TRUE) {
-				dbus_message_iter_get_basic(&value,
-							&idle_timeout);
 			} else {
 				return -EINVAL;
 			}
@@ -1652,9 +1459,6 @@ int __connman_session_create(DBusMessage *msg)
 			if (g_str_equal(key, "ConnectionType") == TRUE) {
 				dbus_message_iter_get_basic(&value, &val);
 				type = string2type(val);
-			} else if (g_str_equal(key, "RoamingPolicy") == TRUE) {
-				dbus_message_iter_get_basic(&value, &val);
-				roaming_policy = string2roamingpolicy(val);
 			} else {
 				return -EINVAL;
 			}
@@ -1704,24 +1508,26 @@ int __connman_session_create(DBusMessage *msg)
 	info = session->info;
 	info_last = session->info_last;
 
+	config_get_bool(owner, "Priority", &priority);
+	config_get_bool(owner, "EmergencyCall", &ecall_app);
+	config_get_string(owner, "RoamingPolicy", &roaming_policy_str);
+	roaming_policy = string2roamingpolicy(roaming_policy_str);
+
 	session->owner = g_strdup(owner);
 	session->session_path = session_path;
 	session->notify_path = g_strdup(notify_path);
 	session->notify_watch =
 		g_dbus_add_disconnect_watch(connection, session->owner,
 					owner_disconnect, session, NULL);
+	if (ecall_app == TRUE)
+		ecall_session = session;
 
 	info->state = CONNMAN_SESSION_STATE_DISCONNECTED;
 	info->type = type;
 	info->priority = priority;
 	info->avoid_handover = avoid_handover;
-	info->stay_connected = stay_connected;
-	info->periodic_connect = periodic_connect;
-	info->idle_timeout = idle_timeout;
-	info->ecall = ecall;
 	info->roaming_policy = roaming_policy;
 	info->entry = NULL;
-	info->marker = 0;
 
 	if (allowed_bearers == NULL) {
 		info->allowed_bearers =
@@ -1757,21 +1563,12 @@ int __connman_session_create(DBusMessage *msg)
 
 
 	populate_service_list(session);
-	if (info->ecall == TRUE) {
-		ecall_info = info;
-		update_ecall_sessions(session);
-	}
 
 	info_last->state = info->state;
 	info_last->priority = info->priority;
 	info_last->avoid_handover = info->avoid_handover;
-	info_last->stay_connected = info->stay_connected;
-	info_last->periodic_connect = info->periodic_connect;
-	info_last->idle_timeout = info->idle_timeout;
-	info_last->ecall = info->ecall;
 	info_last->roaming_policy = info->roaming_policy;
 	info_last->entry = info->entry;
-	info_last->marker = info->marker;
 	info_last->allowed_bearers = info->allowed_bearers;
 
 	session->append_all = TRUE;
@@ -1977,6 +1774,98 @@ static struct connman_notifier session_notifier = {
 	.ipconfig_changed	= ipconfig_changed,
 };
 
+static struct connman_session *session_lookup_by_id(const char *id)
+{
+	struct connman_session *session;
+	GHashTableIter iter;
+	gpointer key, value;
+
+	DBG("id %s", id);
+
+	g_hash_table_iter_init(&iter, session_hash);
+
+	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
+		session = value;
+
+		if (g_strcmp0(session->owner, id) == FALSE)
+			continue;
+
+		return session;
+	}
+
+	DBG("No session found by id %s", id);
+
+	return NULL;
+}
+
+int connman_session_update_bool(const char *id, const char *key,
+					connman_bool_t val)
+{
+	struct connman_session *session;
+	struct session_info *info;
+
+	session = session_lookup_by_id(id);
+	if (session == NULL)
+		return -EINVAL;
+
+	info = session->info;
+	if (info == NULL)
+		return 0;
+
+	DBG("%s %d", key, val);
+
+	return -EINVAL;
+}
+
+int connman_session_update_string(const char *id, const char *key,
+					const char *val)
+{
+	struct connman_session *session;
+	struct session_info *info;
+
+	session = session_lookup_by_id(id);
+	if (session == NULL)
+		return -EINVAL;
+
+	info = session->info;
+	if (info == NULL)
+		return 0;
+
+	DBG("%s %s", key, val);
+
+	return -EINVAL;
+}
+
+int connman_session_config_register(struct connman_session_config *config)
+{
+	DBG("name %s", config->name);
+
+	if (session_config != NULL) {
+		connman_warn("A session configuration plugin '%s' is "
+				"already registerd. Skipping registration "
+				"of plugin '%s'",
+				session_config->name, config->name);
+		return -EALREADY;
+	}
+
+	session_config = config;
+
+	return 0;
+}
+
+void connman_session_config_unregister(struct connman_session_config *config)
+{
+	DBG("name %s", config->name);
+
+	if (config != session_config) {
+		connman_warn("Trying to unregister session configuration "
+				"plugin '%s'", config->name);
+		return;
+	}
+
+	session_config = NULL;
+}
+
 int __connman_session_init(void)
 {
 	int err;
