@@ -575,13 +575,10 @@ static gboolean technology_pending_reply(gpointer user_data)
 	return FALSE;
 }
 
-static int technology_enable(struct connman_technology *technology,
-		DBusMessage *msg)
+static int technology_enable(struct connman_technology *technology)
 {
 	GSList *list;
 	int err = 0;
-	int ret = -ENODEV;
-	DBusMessage *reply;
 
 	DBG("technology %p enable", technology);
 
@@ -596,72 +593,22 @@ static int technology_enable(struct connman_technology *technology,
 		goto done;
 	}
 
-	if (msg != NULL) {
-		/*
-		 * This is a bit of a trick. When msg is not NULL it means
-		 * thats technology_enable was invoked from the manager API.
-		 * Hence we save the state here.
-		 */
-		technology->enable_persistent = TRUE;
-		technology_save(technology);
-	}
-
 	__connman_rfkill_block(technology->type, FALSE);
 
-	/*
-	 * An empty device list means that devices in the technology
-	 * were rfkill blocked. The unblock above will enable the devs.
-	 */
-	if (technology->device_list == NULL) {
-		ret = 0;
-		goto done;
-	}
-
 	for (list = technology->device_list; list; list = list->next) {
 		struct connman_device *device = list->data;
 
 		err = __connman_device_enable(device);
-		/*
-		 * err = 0 : Device was enabled right away.
-		 * If atleast one device gets enabled, we consider
-		 * the technology to be enabled.
-		 */
-		if (err == 0)
-			ret = 0;
 	}
 
 done:
-	if (ret == 0) {
-		if (msg != NULL)
-			g_dbus_send_reply(connection, msg, DBUS_TYPE_INVALID);
-		return ret;
-	}
-
-	if (msg != NULL) {
-		if (err == -EINPROGRESS) {
-			technology->pending_reply = dbus_message_ref(msg);
-			technology->pending_timeout = g_timeout_add_seconds(10,
-					technology_pending_reply, technology);
-		} else {
-			if (err == -EALREADY)
-				reply = __connman_error_already_enabled(msg);
-			else
-				reply = __connman_error_failed(msg, -err);
-			if (reply != NULL)
-				g_dbus_send_message(connection, reply);
-		}
-	}
-
 	return err;
 }
 
-static int technology_disable(struct connman_technology *technology,
-		DBusMessage *msg)
+static int technology_disable(struct connman_technology *technology)
 {
 	GSList *list;
 	int err = 0;
-	int ret = -ENODEV;
-	DBusMessage *reply;
 
 	DBG("technology %p disable", technology);
 
@@ -679,46 +626,55 @@ static int technology_disable(struct connman_technology *technology,
 	if (technology->tethering == TRUE)
 		set_tethering(technology, FALSE);
 
-	if (msg != NULL) {
-		technology->enable_persistent = FALSE;
-		technology_save(technology);
-	}
-
 	__connman_rfkill_block(technology->type, TRUE);
 
 	for (list = technology->device_list; list; list = list->next) {
 		struct connman_device *device = list->data;
 
 		err = __connman_device_disable(device);
-		if (err == 0)
-			ret = 0;
 	}
 
 done:
-	if (ret == 0) {
-		if (msg != NULL)
-			g_dbus_send_reply(connection, msg, DBUS_TYPE_INVALID);
-		return ret;
-	}
-
-	if (msg != NULL) {
-		if (err == -EINPROGRESS) {
-			technology->pending_reply = dbus_message_ref(msg);
-			technology->pending_timeout = g_timeout_add_seconds(10,
-					technology_pending_reply, technology);
-		} else {
-			if (err == -EALREADY)
-				reply = __connman_error_already_disabled(msg);
-			else
-				reply = __connman_error_failed(msg, -err);
-			if (reply != NULL)
-				g_dbus_send_message(connection, reply);
-		}
-	}
-
 	return err;
 }
 
+static DBusMessage *set_powered(struct connman_technology *technology,
+				DBusMessage *msg, connman_bool_t powered)
+{
+	DBusMessage *reply = NULL;
+	connman_bool_t persistent;
+	int err;
+
+	if (powered == TRUE) {
+		err = technology_enable(technology);
+		persistent = TRUE;
+	} else {
+		err = technology_disable(technology);
+		persistent = FALSE;
+	}
+
+	if (err != -EBUSY) {
+		technology->enable_persistent = persistent;
+		technology_save(technology);
+	}
+
+	if (err == -EINPROGRESS) {
+		technology->pending_reply = dbus_message_ref(msg);
+		technology->pending_timeout = g_timeout_add_seconds(10,
+					technology_pending_reply, technology);
+	} else if (err == -EALREADY) {
+		if (powered == TRUE)
+			reply = __connman_error_already_enabled(msg);
+		else
+			reply = __connman_error_already_disabled(msg);
+	} else if (err < 0)
+		reply = __connman_error_failed(msg, -err);
+	else
+		reply = g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+
+	return reply;
+}
+
 static DBusMessage *set_property(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -756,8 +712,12 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		dbus_message_iter_get_basic(&value, &tethering);
 
-		if (technology->tethering == tethering)
-			return __connman_error_already_enabled(msg);
+		if (technology->tethering == tethering) {
+			if (tethering == FALSE)
+				return __connman_error_already_disabled(msg);
+			else
+				return __connman_error_already_enabled(msg);
+		}
 
 		err = set_tethering(technology, tethering);
 		if (err < 0)
@@ -774,6 +734,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 		if (strlen(str) < 1 || strlen(str) > 32)
 			return __connman_error_invalid_arguments(msg);
 
+		g_free(technology->tethering_ident);
 		technology->tethering_ident = g_strdup(str);
 		technology_save(technology);
 	} else if (g_str_equal(name, "TetheringPassphrase") == TRUE) {
@@ -787,6 +748,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 		if (strlen(str) < 8 || strlen(str) > 63)
 			return __connman_error_passphrase_required(msg);
 
+		g_free(technology->tethering_passphrase);
 		technology->tethering_passphrase = g_strdup(str);
 		technology_save(technology);
 	} else if (g_str_equal(name, "Powered") == TRUE) {
@@ -796,11 +758,8 @@ static DBusMessage *set_property(DBusConnection *conn,
 			return __connman_error_invalid_arguments(msg);
 
 		dbus_message_iter_get_basic(&value, &enable);
-		if (enable == TRUE)
-			technology_enable(technology, msg);
-		else
-			technology_disable(technology, msg);
 
+		return set_powered(technology, msg, enable);
 	} else
 		return __connman_error_invalid_property(msg);
 
@@ -932,7 +891,7 @@ static const GDBusMethodTable technology_methods[] = {
 	{ GDBUS_DEPRECATED_METHOD("GetProperties",
 			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
 			get_properties) },
-	{ GDBUS_METHOD("SetProperty",
+	{ GDBUS_ASYNC_METHOD("SetProperty",
 			GDBUS_ARGS({ "name", "s" }, { "value", "v" }),
 			NULL, set_property) },
 	{ GDBUS_ASYNC_METHOD("Scan", NULL, NULL, scan) },
@@ -1006,7 +965,7 @@ static struct connman_technology *technology_get(enum connman_service_type type)
 		return NULL;
 	}
 
-	technology_list = g_slist_append(technology_list, technology);
+	technology_list = g_slist_prepend(technology_list, technology);
 
 	technology_added_signal(technology);
 
@@ -1045,6 +1004,8 @@ static void technology_put(struct connman_technology *technology)
 
 	g_free(technology->path);
 	g_free(technology->regdom);
+	g_free(technology->tethering_ident);
+	g_free(technology->tethering_passphrase);
 	g_free(technology);
 }
 
@@ -1149,7 +1110,7 @@ int __connman_technology_add_device(struct connman_device *device)
 	if (!technology->enable_persistent)
 		__connman_device_disable(device);
 
-	technology->device_list = g_slist_append(technology->device_list,
+	technology->device_list = g_slist_prepend(technology->device_list,
 								device);
 
 	return 0;
@@ -1267,10 +1228,10 @@ int __connman_technology_set_offlinemode(connman_bool_t offlinemode)
 		struct connman_technology *technology = list->data;
 
 		if (offlinemode)
-			err = technology_disable(technology, NULL);
+			err = technology_disable(technology);
 
 		if (!offlinemode && technology->enable_persistent)
-			err = technology_enable(technology, NULL);
+			err = technology_enable(technology);
 	}
 
 	if (err == 0 || err == -EINPROGRESS || err == -EALREADY) {
